/*
*********************************************
*											*
*		UTC TAI sample program				*
*											*
*********************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "atime.h"

#define		VERSION			"atime Ver 2016/10/20\n"
#define		REAPSEC_FILE	"dAT.tbl"

LEAPSEC *pdAT;
char *pszLJD = "JD";
char *pszSJD = "jd";

/*
=============================================
=		display help						=
=============================================
*/
void disp_help(void)
{
	puts(VERSION);
	puts("atime --- PC(UTC) system time");
	puts("atime Year [Month Day Hour Minute Second] --- Any UTC time");
	puts("atime jd JD-DATA --- Any JD");
}

/*
=============================================
=		Read leapsec. table for TT			=
=============================================
*/
int read_leapsec_table(char *pFileName)
{
	FILE *fp;
	LEAPSEC *ptbl;

	unsigned long int ndTCount;
	double sec;
	char szDmy[80];

	/*
	-------------------------------------------
			check parameter line
	-------------------------------------------
	*/
	/* Open File */
	fp = fopen(pFileName, "r");
	if(fp == NULL){
		printf("%s cannot open\n", pFileName);
		return -1;
	}
	/* calc. size */
	ndTCount = 0;
	while(!feof(fp)){
		if (fgets(szDmy, sizeof(szDmy), fp) == NULL){
			break;
		}
		ndTCount++;
	}

	/* memory allocate */
	pdAT = (LEAPSEC *)malloc(ndTCount * sizeof(LEAPSEC));
	if (pdAT == NULL){
		fclose(fp);
		puts("Memory allocate error!");
		return -1;
	}

	/*
	-------------------------------------------
			load reap second data
	-------------------------------------------
	*/
	/* Read data */
	ptbl = pdAT;
	fseek(fp, 0, SEEK_SET);
	while(!feof(fp)){
		if (fgets(szDmy, sizeof(szDmy), fp) == NULL){
			break;
		}
		ptbl->dJD = atof(szDmy);
		sec = atof(&szDmy[10]);
		ptbl->dTime = (sec + DTAI_TT) / DAYSEC;
		ptbl++;
	}
	fclose(fp);

	return 0;
}

int main(int argc, char *argv[])
{
	time_t timer;
	struct tm *t_st;

	DATETIME dtime = {{0, 1, 1, 0.0}, {0, 0, 0.0}};
	LONGJD dJD_TT;
	LONGJD dJD;
	double dDAT;
	char *pHead;
	char cTemp;
	int nTTMode;
	int i;
	int bJDFlag = 0;

	/* headder */
	char szDT[] = "dT(ET-UT)";
	char szET[] = "ET ";
	char szUT[] = "UT ";

	char szDTT[] = "dTT(TT-UTC)";
	char szTT[] = "TT ";
	char szUTC[] = "UTC";

	/* Set arg data */
	if (argc == 1){
		/* get now time */
		time(&timer);
		t_st = gmtime(&timer);
		dtime.date.dYear = t_st->tm_year+1900;
		dtime.date.nMonth = t_st->tm_mon+1;
		dtime.date.nDay = t_st->tm_mday;
		dtime.time.nHour = t_st->tm_hour;
		dtime.time.nMin = t_st->tm_min;
		dtime.time.dSec = t_st->tm_sec;
	}else{
		i = argc - 1;
		if (i != 0){
			/* check help(-h or -?) */
			if (argv[1][0] == '-'){
				cTemp = argv[1][1];
				if ((cTemp == 'h') || (cTemp == 'H') || (cTemp == '?')){
					disp_help();
					return -1;
				}
			}
			/* check JD input */
			if ((strcmp(argv[1], pszLJD) == 0) || (strcmp(argv[1], pszSJD) == 0)){
				bJDFlag = -1;
			}else{
				dtime.date.dYear = atof(argv[1]);
			}
			i--;
		}
		if (i != 0){
			/* check JD input */
			if (bJDFlag != 0){
				dtime.date.dYear = atof(argv[2]);
				i = 0;
			}else{
				dtime.date.nMonth = atoi(argv[2]);
				i--;
			}
		}
		if (i != 0){
			dtime.date.nDay = atoi(argv[3]);
			i--;
		}
		if (i != 0){
			dtime.time.nHour = atoi(argv[4]);
			i--;
		}
		if (i != 0){
			dtime.time.nMin = atoi(argv[5]);
			i--;
		}
		if (i != 0){
			dtime.time.dSec = atof(argv[6]);
			i--;
		}
	}

	/* DAT table read */
	pdAT = NULL;
	if (read_leapsec_table(REAPSEC_FILE) != 0){
		return -1;
	}
	set_dtt_table(pdAT);

	/* check JD input */
	if (bJDFlag != 0){
		jdtoljd(dtime.date.dYear, &dJD);
		juian_dtime_ljd(&dJD, &dtime);
	}
	/* display input time */
	nTTMode = 0;
	if (dtime.date.dYear >= 1972.0){
		nTTMode = -1;
	}

	if (nTTMode != 0){
		pHead = szUTC;
	}else{
		pHead = szUT;
	}
	printf("%s %04.0f/%02d/%02d %02d:%02d:%09.6f\n", pHead, dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);
	printf("JD  %f\n\n", dtime_julian(&dtime));

	/* UTC to TT or ET */
	dDAT = utc_tt(&dtime, &dJD_TT);
	if (nTTMode != 0){
		pHead = szDTT;
	}else{
		pHead = szDT;
	}
	printf("%s %f\n", pHead, dDAT);

	/* display TT date & time */
	juian_dtime_ljd(&dJD_TT, &dtime);
	if (nTTMode != 0){
		pHead = szTT;
	}else{
		pHead = szET;
	}
	printf("%s %04.0f/%02d/%02d %02d:%02d:%09.6f\n", pHead, dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	/* TCG */
	dDAT = tt_tcg(&dJD_TT, &dJD);
	printf("dTCG(TCG-TT) %f\n", dDAT);

	juian_dtime_ljd(&dJD, &dtime);
	printf("TCG %04.0f/%02d/%02d %02d:%02d:%09.6f\n\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	tcg_tt(&dJD, &dJD_TT);
	juian_dtime_ljd(&dJD_TT, &dtime);
	printf("TT  %04.0f/%02d/%02d %02d:%02d:%09.6f\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	/* TT to UTC */
	tt_utc(&dJD_TT, &dtime);
	if (nTTMode != 0){
		pHead = szUTC;
	}else{
		pHead = szUT;
	}
	printf("%s %04.0f/%02d/%02d %02d:%02d:%09.6f\n", pHead, dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	/* Memory free */
	free(pdAT);
	return 0;
}
